#!/bin/bash
# ******************************************************************************
# * FILE PURPOSE: PDK Unit Test and Example Project Creator
# ******************************************************************************
# * FILE NAME: pdkProjectCreate.bat
# *
# * DESCRIPTION: 
# *  The script file is used to create the test and example projects of all
# *  components under PDK. These projects are available in the specified 
# *  workspace.
# *
# * USAGE:
# *  ./pdkProjectCreate.sh [deviceName] [endian] [module] [pdkDir] 
# *
# *Description:     (first option is default)
# *  deviceName  -   k2k / k2h 
# *  endian      -   little / big
# *  module      -   bcp / tcp3d / qmss , etc
# *  pdkDir      -   THIS FILE LOCATION / "C:\ti\pdk_keystone2_<version_no>\packages"
# *
# * Example:
# *
# *  a) ./pdkProjectCreate.sh
# *              - Creates all module projects for k2k device for little endian
# *  b) ./pdkProjectCreate.sh k2h
# *              - Creates all module projects for k2h device for little endian
# *  c) ./pdkProjectCreate.sh k2k big
# *              - Creates all module projects for k2k device for big endian
# *  d) ./pdkProjectCreate.sh k2k big qmss
# *              - Creates qmss projects for k2k device for big endian
# *
# * Copyright (C) 2012-2014, Texas Instruments, Inc.
# *****************************************************************************

# Parameter Validation: Check if the argument was passed to the batch file.
# *****************************************************************************
# Argument [deviceName] is used to set DEVICE_NAME variable.
# Valid values are 'k2k', 'k2h'. Defaults to 'k2k'.
tempVar1=$1
if [ ! -z $tempVar1 ]; then
export DEVICE_NAME=$tempVar1
else
export DEVICE_NAME=k2k
fi

# Argument [endian] is used to set ENDIAN variable.
# This is Endianess of the Projects being created.
# Valid Values are 'little' and 'big'. Defaults to 'little'.
tempVar2=$2
if [ ! -z $tempVar2 ]; then
export ENDIAN=little
if [ "$tempVar2" = "big" ]; then
export ENDIAN=big
fi
fi

# Argument [module] is used to set MODULE variable.
# This is specific module for which Projects are created.
# Valid Values are LLD name. Defaults to all LLD's.

tempVar3=$3
MODULE=all
if [ ! -z $tempVar3 ]; then
MODULE=$tempVar3
fi

# Argument [pdkDir] is used to set PDK_SHORT_NAME. This is PDK directory
# where project description files are located. If there is no value passed,
# it defaults to the file location directory. Then convert the variable to 
# short name to avoid issues using batch file commands.
tempVar4=$4
export PDK_SHORT_NAME=$PWD
if [ ! -z $tempVar4 ]; then
export PDK_SHORT_NAME=$tempVar4
fi

# *****************************************************************************

echo =========================================================================
echo .   DEVICE_NAME     :   $DEVICE_NAME
echo .   ENDIAN          :   $ENDIAN
echo .   MODULE          :   $MODULE
echo .   PDK_SHORT_NAME  :   $PDK_SHORT_NAME
echo =========================================================================

# *****************************************************************************
# * Version Information of the various tools etc required to build the test
# * projects. Customers are free to modify these to meet their requi#ents.
# *****************************************************************************

# This is to control the CCS version specific project create command
# export to 'no' when using CCSv5 or export to 'yes' when using CCSv4
export IS_CCS_VERSION_4=no

# export to 'no' when using QT, EVM, VDB, or other hardware. export to 'yes' only when using the simulator.
export IS_SIMULATOR_SUPPORT_NEEDED=no

# Install Location for CCS
export CCS_INSTALL_PATH=~/ti/ccsv5

# Workspace where the PDK projects will be created.
export MY_WORKSPACE=$PDK_SHORT_NAME/exampleProjects

# macros.ini location
export MACROS_FILE=$PDK_SHORT_NAME/macros.ini

# This is the format of the executable being created
# Valid Values are 'ELF' and 'COFF'
export OUTPUT_FORMAT=ELF

# Version of CG-Tools
export CGT_VERSION=7.4.4

# Version of XDC
export XDC_VERSION=3.25.05.94

# Version of BIOS
export BIOS_VERSION=6.37.00.20

# Version of the IPC
export IPC_VERSION=3.00.04.29

# EDMA3 Version
export EDMA_VERSION=

# Version of the PDK
export PDK_VERSION=3.00.04.18

# Version of the SA LLD
export SA_VERSION=2.00.03.00

# Version of UIA
export UIA_VERSION=1.03.02.10

# PDK Part Number
export PDK_PARTNO=TCI6634

# PDK Part Number & Platform name
if [ "$DEVICE_NAME" = "k2k" ]; then
export PDK_PARTNO=TCI6638
export RTSC_PLATFORM_NAME=ti.platforms.evmTCI6638K2K
else
if [  "$DEVICE_NAME" = "k2h" ]; then
export PDK_PARTNO=TCI6636
export RTSC_PLATFORM_NAME=ti.platforms.evmTCI6636K2H
else
export PDK_PARTNO=TCI6634
export RTSC_PLATFORM_NAME=ti.platforms.simKepler
fi
fi

# RTSC Target 
# - Please ensure that you select this taking into account the
#   OUTPUT_FORMAT and the RTSC_PLATFORM_NAME 
if [ "$ENDIAN" = "big" ]; then
export RTSC_TARGET=ti.targets.elf.C66_big_endian
else
export RTSC_TARGET=ti.targets.elf.C66
fi

# *****************************************************************************
# *****************************************************************************
#                 Please do NOT change anything below this
# *****************************************************************************
# *****************************************************************************

# export auto create command by default for use with CCSv5
export AUTO_CREATE_COMMAND=eclipse/ccstudio

# If is CCS version 4 then export auto create command for use with CCSv4
# If .%IS_CCS_VERSION_4% == .yes export AUTO_CREATE_COMMAND=eclipse\jre\bin\java -jar $CCS_INSTALL_PATH\eclipse\startup.jar

# export project for Silicon or QT by default
export SIMULATOR_SUPPORT_DEFINE=

ccs_create()
{
	echo found $1
	textfile=${1#*/}
	$CCS_INSTALL_PATH/$AUTO_CREATE_COMMAND -nosplash -data $MY_WORKSPACE -application com.ti.ccstudio.apps.projectCreate -ccs.name ${textfile%.*} -ccs.outputFormat $OUTPUT_FORMAT -ccs.device com.ti.ccstudio.deviceModel.C6000.GenericC66xxDevice -ccs.endianness $ENDIAN -ccs.kind executable -ccs.cgtVersion $CGT_VERSION -rtsc.xdcVersion $XDC_VERSION -rtsc.enableDspBios -rtsc.biosVersion $BIOS_VERSION -rtsc.buildProfile "debug" -rtsc.products "com.ti.sdo.edma3:$EDMA_VERSION;com.ti.rtsc.IPC:$IPC_VERSION;com.ti.rtsc.SYSBIOS:$BIOS_VERSION;ti.pdk:$PDK_VERSION;ti.drv.sa:$SA_VERSION;com.ti.uia:$UIA_VERSION" -rtsc.platform "$RTSC_PLATFORM_NAME" -rtsc.target $RTSC_TARGET -ccs.rts libc.a -ccs.args $textfile \;
	cp $PDK_SHORT_NAME/macros.ini $MY_WORKSPACE/${textfile%.*}
}

export -f ccs_create

find -iname "*${DEVICE_NAME}*roject.txt" -execdir bash -c 'ccs_create {}' \;

