/******************************************************************************
 * FILE PURPOSE: Main routine for exercising SA Unit Tests.
 *
 ******************************************************************************
 * FILE NAME:    testMain.c
 *
 * DESCRIPTION:  The SA unit tests are designed to veriy the SASS and SA LLD
 *               functionalities for all supported security protocols such as 
 *               IPSEC, SRTP and etc. Please refer to the SA LLD test plan and 
 *               the description in each test file for details. 
 *
 *               It contains the following files and directories:
 *
 *  testMain.c:  Primary test routines and the test table
 *  testconn.c:  System routines to establish connections and packet generation
 *               and other common test functionalities
 *  testmem.c:   Static memory allocation for test framework 
 *  testutil.c:  Utility functions
 *  framework.c: Functions to setup system, CPPI, QMSS, PA and SA to establish
 *               test frameworks
 *  common.c:    Common system functions used by PA and SA
 *  salldsim:    Directory contains SA LLD API calls and configuration files
 *  test:        Directory contains the test files for all test cases          
 *
 *  Configurations: Enable/Disable Test log
 *      Global variable saTestLogEnable should be set or clear as desribed below.
 *  
 * (C) Copyright 2009-2012, Texas Instruments, Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "unittest.h"
#include "testconn.h"

/* Enable/Disable SA Unit Test log display */
Bool saTestLogEnable = FALSE;

/* 
 * Enable/Disable Simulator:
 *  
 * TRUE: simulator is used 
 * FALSE: silicon is used
 *
 * Note: This varaible is used to determine some test parameters such as number of test packets,
 *       number of test channels to avoid the long test time at simulator.
 */
#ifdef SIMULATOR_SUPPORT
Bool saSimTest = TRUE;
#else
Bool saSimTest = FALSE;
#endif


/* NULL terminated The list of tests */
saTest_t  saTestList[] = {
	{ saESPTest,          "SA IPSEC ESP test",                   SA_TEST_NOT_RUN },
    { saAHTest,           "SA IPSEC AH test",                    SA_TEST_NOT_RUN },
	{ saESPNATTTest,      "SA IPSEC ESP NAT-T test",             SA_TEST_NOT_RUN },
    { saSRTPTest,         "SA SRTP test",                        SA_TEST_NOT_RUN },
    { saSRTCPTest,        "SA SRTCP test",                       SA_TEST_NOT_RUN },
    { saSrtpEspTest,      "SA SRTP over ESP test",               SA_TEST_NOT_RUN },
    { saSrtpAhTest,       "SA SRTP over AH test",                SA_TEST_NOT_RUN },
	{ saESPMultiSgmtTest, "SA IPSEC ESP Multi-segment test",     SA_TEST_NOT_RUN },
	{ saAHMultiSgmtTest,  "SA IPSEC AH Multi-segment test",      SA_TEST_NOT_RUN },
	{ saSRTPMultiSgmtTest,"SA SRTP Multi-segment test", 	     SA_TEST_NOT_RUN },
	{ saSRTCPMultiSgmtTest,"SA SRTCP Multi-segment test", 	     SA_TEST_NOT_RUN },
    #ifdef SA_3GPP_SUPPORT
	{ saAirCipheringTest, "SA 3GPP Air Ciphering test",          SA_TEST_NOT_RUN },
	{ saAirCipheringKeyGenTest, "SA 3GPP Air Ciphering Key stream generation test",          SA_TEST_NOT_RUN },
    #endif
	{ saSRTPRekeyTest1,   "SA SRTP Rekey test 1",                SA_TEST_NOT_RUN },
	{ saSRTPRekeyTest2,   "SA SRTP Rekey test 2",                SA_TEST_NOT_RUN },
	{ saSRTPRekeyTest3,   "SA SRTP Rekey test 3",                SA_TEST_NOT_RUN },
	{ saSRTPReKeyTest4,   "SA SRTP Rekey test 4",           	 SA_TEST_NOT_RUN },
	{ saSRTCPRekeyTest1,  "SA SRTCP Rekey test 1",               SA_TEST_NOT_RUN },
	{ saSRTCPRekeyTest2,  "SA SRTCP Rekey test 2",               SA_TEST_NOT_RUN },
	{ saESPReplayTest,    "SA IPSEC ESP Replay test",            SA_TEST_NOT_RUN },
    /* There are no PA stats check for the following tests due to errors expected */
	{ saAHReplayTest,     "SA IPSEC AH Replay test",             SA_TEST_NOT_RUN },
	{ saSRTPReplayTest,   "SA SRTP Replay test",                 SA_TEST_NOT_RUN },
	{ saSRTCPReplayTest,  "SA SRTCP Replay test",                SA_TEST_NOT_RUN },
	{ saESPErrTest,       "SA IPSEC ESP Error Handling test",    SA_TEST_NOT_RUN },
    { saAHErrTest,        "SA IPSEC AH Error Handling test",     SA_TEST_NOT_RUN },
    { saSRTPErrTest,      "SA SRTP Error Handling test",         SA_TEST_NOT_RUN },
    { saSRTCPErrTest,     "SA SRTCP Error Handling test",        SA_TEST_NOT_RUN },
	{ saDataModeTest,     "SA Data Mode test",                   SA_TEST_NOT_RUN },
	{ saRngTest,          "SA TRNG test",                        SA_TEST_NOT_RUN },
    #ifndef SIMULATOR_SUPPORT
	{ saPkaTest,          "SA PKA test",                         SA_TEST_NOT_RUN },
    #endif
	{ NULL,               NULL,                                  SA_TEST_NOT_RUN }
};


Void topLevelTest (UArg a0, UArg a1);


/* The exit code is a global. This is used so
 * the clock function can terminate the program with
 * the proper exit code */
Int exitCode;

paSysStats_t paTestExpectedStats;  /* Expected stats results */

t2Handles_t     l2Handles[SA_TEST_MAX_L2_HANDLES];
t2Handles_t     l3Handles[SA_TEST_MAX_L3_HANDLES];
t4Handles_t     l4Handles[SA_TEST_MAX_L4_HANDLES];
int numL2Handles, numL3Handles, numL4Handles;
									    
Void saTestRecoverAndExit (tFramework_t *tf, saTest_t *pat, saTestStatus_t testResult)
{
	Cppi_HostDesc *hd;
    Int numCon;
	
	/* Set the result code */
	pat->testStatus = testResult;
	
	/* Restore the packet recycle queues */
	while (Qmss_getQueueEntryCount ((tf->QGen)[Q_DATA_RECYCLE]))  {
		
		hd = (Cppi_HostDesc *)(((UInt32)Qmss_queuePop (tf->QGen[Q_DATA_RECYCLE])) & ~15);
		
		hd->buffPtr = hd->origBuffPtr;
		hd->buffLen = hd->origBufferLen;
		
		Qmss_queuePushDesc (tf->QfreeDesc, (Ptr)hd);
	}
    
    /* Zero out the expected stats. The stats will be updated as packets are sent into PA */
    memset (&paTestExpectedStats, 0, sizeof(paTestExpectedStats));
    
    /* Delete L2/L3/l4 Handles */
    if (!testCommonDelL4Connection(tf, pat, numL4Handles, l4Handles, &numCon))
    {
        pat->testStatus = SA_TEST_FAILED;
    }
    
    if (testCommonDelL2L3Connection(tf, pat, numL3Handles, l3Handles, &numCon))
    {
        paTestExpectedStats.classify1.nPackets += numCon;    
    }
    else
    {
        pat->testStatus = SA_TEST_FAILED;
    }
    
    if (testCommonDelL2L3Connection(tf, pat, numL2Handles, l2Handles, &numCon))
    {
        paTestExpectedStats.classify1.nPackets += numCon;    
    }
    else
    {
        pat->testStatus = SA_TEST_FAILED;
    }
	
    /* Close all SA LLD channels */
    salldSim_close_all_chns();
    
	/* Return */
    #ifdef USE_BIOS
	Task_exit();
    #else
    if (pat->testStatus == SA_TEST_FAILED)
    {
	    Task_exit();
    }
    #endif
}

static Void saTestInit(Void)
{
    /* Zero out the handle arrays and packet counts */
    memset (l2Handles, 0, sizeof(l2Handles));
    memset (l3Handles, 0, sizeof(l3Handles));
    memset (l4Handles, 0, sizeof(l4Handles));
    numL2Handles = 0;
    numL3Handles = 0;
    numL4Handles = 0;
    
    /* Zero out all connection realted handles */
    numHdrHandles = 0;
    numConnHandles = 0;
    numSrtpHandles = 0;
    numIpsecHandles = 0;
    
    /* Zero out SA LLD channel instances */
    numSalldSimChans = 0;
}    

/* Creates a single task - the top level task. This is a low priority task that
 * spawns the individual tests */
#ifndef ARM11 
extern cregister volatile unsigned int TSCL;
#endif
Void main ()
{
#ifdef USE_BIOS
	Task_Params tparams;
	
	/* The only initial task is the top level test */
	Task_Params_init (&tparams);
	tparams.instance->name = "Top Level Test";
	tparams.priority      = 1;
	
	Task_create (topLevelTest, &tparams, NULL);
	
	/* Start the cycle counter */
	TSCL = 1;
	
	BIOS_start ();
    
#else
	/* Start the cycle counter */
#ifndef ARM11 
	TSCL = 1;
#endif    

    topLevelTest(NULL, NULL);
#endif    
}

Void clk1Fxn (UArg a0)
{
	BIOS_exit (exitCode);
}

UInt16 numTestPkt1 = 200;
UInt16 numTestPkt2 = 40;
UInt16 numTestPkt3 = 40;

/* Initialize the test framework and launch the individual tests */
Void topLevelTest (UArg a0, UArg a1)
{
#ifdef USE_BIOS
	Task_Params tparams;
	Task_Handle thandle;
	Task_Stat   tstat;
	
	Clock_Handle clkh;       
    Clock_Params clkParams;
#endif    
	Int i;
	
	Int passCount;
	Int failCount;
	Int notRunCount;
	
#ifdef USE_BIOS    
	/* For some reason some printfs are lost unless there is a delay between System_flush
	 * and System_exit, so delay is forced */	
	Clock_Params_init(&clkParams);
    clkParams.period    = 0;
    clkParams.startFlag = FALSE;
    clkh = Clock_create(clk1Fxn, 1, &clkParams, NULL);
    
    
#endif  

    /* Disable L1 and L2 Cache */
    //CACHE_wbAllL1d (CACHE_WAIT);
    //CACHE_setL1DSize(CACHE_L1_0KCACHE);
    //CACHE_setL1PSize(CACHE_L1_0KCACHE);
    CACHE_setL2Size(CACHE_0KCACHE);
  
    
    System_printf ("\n\n ------- SA Unit Test Starting ---------\n");
    System_flush ();
    
    if (saSimTest)
    {
        /* reduce the number of test packets to shorten test time */
        numTestPkt1 = 10;
        numTestPkt2 = 10;
        numTestPkt3 = 10;
    }
	
    /* Initialize the PA, PA cpdma, QM and CPPI. Each test will use
     * the same framework */
    if (setupTestFramework ())  {
    	System_printf ("topLevelTest (%s:%d): setupTestFramework returned error, exiting\n", __FILE__, __LINE__);
    	System_flush ();
    	exitCode = -1;
        #ifdef USE_BIOS
    	Clock_start(clkh);
        #endif
    	Task_exit ();
    }
    
    /* Make sure the setup matches what is expected */
    if (verifyTestFramework())  {
    	System_printf ("topLevelTest (%s:%d): verifyTestFramework returned error after initial framework setup, exiting\n", __FILE__, __LINE__);
    	System_flush();
    	exitCode = -1;
        #ifdef USE_BIOS
    	Clock_start(clkh);
        #endif
    	Task_exit ();
    }
    
    /* Zero out the expected stats. The stats will be updated as packets are sent into PA */
    memset (&paTestExpectedStats, 0, sizeof(paTestExpectedStats));
     
#ifdef USE_BIOS     
    /* Configure task parameters common to all test tasks */
    Task_Params_init (&tparams);
    tparams.priority = 2;
    tparams.arg0     = (UArg) &tFramework;
#endif
    
	/* Run the tests */
	for (i = 0; saTestList[i].testFunction != NULL; i++ )  {
		
        saTestInit();
        utilPrepFout(&tFramework);
        
        #ifdef USE_BIOS
		tparams.arg1 = (UArg)&saTestList[i];
        tFramework.pTest = &saTestList[i];
		tparams.instance->name = saTestList[i].name;
		
		thandle = Task_create (saTestList[i].testFunction, &tparams, NULL);
        
		/* The test task will terminate upon completion. Verify that the
		 * task has completed in case the task itself uses multiple tasks
		 * that all wind up idle for a while. */
		do  {
			Task_stat (thandle, &tstat);
		} while (tstat.mode != Task_Mode_TERMINATED);
        
        Task_delete (&thandle);
        
        #else
        /* Just call the test function */
        saTestList[i].testFunction((UArg)&tFramework, (UArg)&saTestList[i]);
        #endif
		
		if (saTestList[i].testStatus == SA_TEST_PASSED)
		  System_printf ("%s:  PASSED\n", saTestList[i].name);
		else
		  System_printf ("%s:  FAILED\n", saTestList[i].name);
          
        System_flush ();
		
		/* Do a quick check of the test framework */
		if (verifyTestFramework ())  {
			System_printf ("topLevelTest (%s:%d): verifyTestFramework returned error after test %s. Exiting.\n", __FILE__, __LINE__, saTestList[i].name);
			exitCode = -1;
			System_flush ();
            #ifdef USE_BIOS
			Clock_start(clkh);
            #endif
		}
        
        utilCloseFout(&tFramework);
	} 
		
	
	/* Summarize the test results */
	for (i = passCount = failCount = notRunCount = 0; saTestList[i].testFunction != NULL; i++)  {
		if (saTestList[i].testStatus == SA_TEST_PASSED)
			passCount += 1;
		else if (saTestList[i].testStatus == SA_TEST_FAILED)
			failCount += 1;
		else
			notRunCount += 1;
	}
	
	System_printf ("\n\nTest summary:\n\tTests Passed: %d\n\tTests Failed: %d\n\tTests not run: %d\n\n",
	  passCount, failCount, notRunCount);

	System_printf ("\n\n ------- SA Unit Test Complete ---------\n");
	System_flush();
    #ifdef USE_BIOS
	Clock_start(clkh);
    #endif
	
	Task_exit();
}




