/*
 *
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "../unittest.h"
#include "../salldsim/salldsim.h"

/* TRNG (True Random Number Generator) test.
 * This test is designed to verify the opertions of the True Random Number
 * Generator (TRNG) within SASS.
 *
 * Test Procedures:
 * 
 * - Configure and enable TRNG with all default parameters
 *   - Clock Divider = 1
 *   - Startup Cycle = 512 (2**8)
 *   - Minimum Refill Cycle = 64 (2**6)
 *   - Maxmium Refill Cycle = 2**24
 *
 * - Extract (and display) 64-bit random number every 200 CPU cycles
 * - Report number of random number received per 100 tries
 * 
 * a0 is a pointer to the test framework structure
 * a1 is a pointer to the saTest_t structure for this test, as initialized in testMain.
 * 
 */
 
#define  SA_RNG_NUM_RANDOM_NUMBER       100

/* SA TRNG test */
Void saRngTest (UArg a0, UArg a1)
{
 	tFramework_t  *tf  = (tFramework_t *)a0;
 	saTest_t      *pat = (saTest_t *)a1;
    Sa_RngConfigParams_t rngCfg;
    Sa_RngData_t rngData[SA_RNG_NUM_RANDOM_NUMBER];
 	Int  i, numRn;
    int16_t retCode;
    
    /* Initialize  TRNG */
    memset(&rngCfg, 0, sizeof(Sa_RngConfigParams_t));
    rngCfg.ctrlBitfield = sa_RNG_CTRL_REINIT;   /* Force re-initialize */
    rngCfg.clockDiv = 0; /* default (1) */
    rngCfg.startupCycles = 0; /* default (2**8) */
    rngCfg.minRefillCycles = 0; /* default (2**6) */
    rngCfg.maxRefillCycles = 0; /* default (2**24) */
    
    /* Configure the TRNG module to generate True random number  */
    retCode = Sa_rngInit(tf->salld_handle, &rngCfg);
    
    if (retCode != sa_ERR_OK)
    {
        salld_sim_disp_control(TRUE);    
        salld_sim_print("saRngTest: Sa_rngInit() returns error code = %d!\n", retCode);
        salld_sim_disp_control(FALSE);    
        
        saTestRecoverAndExit (tf, pat, SA_TEST_FAILED);  /* no return */
        
    }
    
    /* Wait for the first random number */
	utilCycleDelay (10000);
    
    /* Random Number Test */
    for (i = 0, numRn = 0; i < 100; i++)
    {
        if (Sa_getRandomNum (tf->salld_handle, FALSE, &rngData[numRn]) == sa_ERR_OK)
        {
            numRn++;
        }   
		utilCycleDelay (200);
    }    
    
    /* Display the random numbers */
    for (i = 0;  i < numRn; i++)
    {
        salld_sim_print("saRngTest: Random Number %d = 0x%08x%08x!\n", i, rngData[i].hi, rngData[i].lo);
    }
    
    salld_sim_disp_control(TRUE);    
    salld_sim_print("saRngTest: %d (out of %d) Random Number are generated!\n", numRn, SA_RNG_NUM_RANDOM_NUMBER);
    salld_sim_disp_control(FALSE);    
    
 	saTestRecoverAndExit (tf, pat, SA_TEST_PASSED);  /* no return */
}
	
	
	
		
	


  		
