
#ifndef _UNITTEST_H
#define _UNITTEST_H

/******************************************************************************
 * FILE PURPOSE: Definitions and prototypes for SA Unit Test.
 ******************************************************************************
 * FILE NAME:   unittest.h
 *
 * DESCRIPTION: Contains definitions and function prototypes for SA
 *              Unit Test.
 *
 * (C) Copyright 2009 Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <xdc/std.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Memory.h>
#include <xdc/runtime/Error.h>

#ifdef USE_BIOS
#include <ti/sysbios/bios.h>
#include <ti/sysbios/knl/task.h>
#include <ti/sysbios/knl/Semaphore.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/sysbios/hal/Hwi.h>
#include <ti/sysbios/gates/GateHwi.h> 
#endif

#include <ti/drv/pa/pa.h>
#include <ti/drv/pa/pasahost.h>   
#include <ti/drv/qmss/qmss_drv.h>
#include <ti/drv/cppi/cppi_drv.h>
#include <ti/drv/cppi/cppi_desc.h>
#include <ti/csl/csl_cacheAux.h>
#include <ti/csl/csl_xmcAux.h>

#include <string.h>
#include "salldsim/salldsim.h"

/* Due to a bug in the simulator all packet lengths must be rounded up to 4 byte multiples */
#define PA_SIM_BUG_4BYTES

/* 
 * Compiler Switch for Test Vector Collections 
 *
 * Note:We only support test vector collection operation for single-channel single-mode operation
 */
#define SA_GEN_TEST_VECTOR_TX       0
#define SA_GEN_TEST_VECTOR_RX       0
#define SA_GEN_TEST_VECTOR   (SA_GEN_TEST_VECTOR_TX || SA_GEN_TEST_VECTOR_RX)

typedef enum  {
	SA_TEST_FAILED  = -1,
	SA_TEST_NOT_RUN,
	SA_TEST_PASSED
} saTestStatus_t;

/* Define the test interface */
typedef struct saTest_s
{
	void (*testFunction)(UArg, UArg);   /* The code that runs the test */
	char *name;							/* The test name */
	saTestStatus_t testStatus;			/* Test status */
	
} saTest_t;

/* Number of cpdma channels is fixed by device */
#define TF_PA_NUM_RX_CPDMA_CHANNELS 24
#define TF_PA_NUM_TX_CPDMA_CHANNELS  9
#define TF_PA_TX_QUEUE_INDEX_PDSP0          0
#define TF_PA_TX_QUEUE_INDEX_PDSP1          1
#define TF_PA_TX_QUEUE_INDEX_PDSP2          2
#define TF_PA_TX_QUEUE_INDEX_PDSP3          3
#define TF_PA_TX_QUEUE_INDEX_PDSP4          4
#define TF_PA_TX_QUEUE_INDEX_PDSP5          5
#define TF_PA_TX_QUEUE_INDEX_SA0            6
#define TF_PA_TX_QUEUE_INDEX_SA1            7
#define TF_PA_TX_QUEUE_INDEX_ENET0          8
#define TF_PA_TX_QUEUE_INDEX_ENET1          9
#ifndef ARM11
#define TF_PA_TX_QUEUE_BASE                 640
#define CONFIG_FIRST_GEN_QUEUE              899
#define TF_NUM_GEN_QUEUES		   			100
#else
#define TF_PA_TX_QUEUE_BASE                 0
#define CONFIG_FIRST_GEN_QUEUE              33
#define TF_NUM_GEN_QUEUES		   			20
#endif
/* PA Tx Queues */
#define TF_PA_TX_QUEUE_PDSP0                (TF_PA_TX_QUEUE_BASE + TF_PA_TX_QUEUE_INDEX_PDSP0)
#define TF_PA_TX_QUEUE_PDSP1                (TF_PA_TX_QUEUE_BASE + TF_PA_TX_QUEUE_INDEX_PDSP1)
#define TF_PA_TX_QUEUE_PDSP2                (TF_PA_TX_QUEUE_BASE + TF_PA_TX_QUEUE_INDEX_PDSP2)
#define TF_PA_TX_QUEUE_PDSP3                (TF_PA_TX_QUEUE_BASE + TF_PA_TX_QUEUE_INDEX_PDSP3)
#define TF_PA_TX_QUEUE_PDSP4                (TF_PA_TX_QUEUE_BASE + TF_PA_TX_QUEUE_INDEX_PDSP4)
#define TF_PA_TX_QUEUE_PDSP5                (TF_PA_TX_QUEUE_BASE + TF_PA_TX_QUEUE_INDEX_PDSP5)

/* SA (CP_ACE) queues */
#define TF_SA_QUEUE_0                       (TF_PA_TX_QUEUE_BASE + TF_PA_TX_QUEUE_INDEX_SA0)
#define TF_SA_QUEUE_1                       (TF_PA_TX_QUEUE_BASE + TF_PA_TX_QUEUE_INDEX_SA1)

/* ENET queues */
#define TF_ENET_QUEUE_0                     (TF_PA_TX_QUEUE_BASE + TF_PA_TX_QUEUE_INDEX_ENET0)
#define TF_ENET_QUEUE_1                     (TF_PA_TX_QUEUE_BASE + TF_PA_TX_QUEUE_INDEX_ENET1)


#define TF_Q_FREE_DESC						(CONFIG_FIRST_GEN_QUEUE + 1)   /* Unassociated descriptor queue number */
#define TF_TEARDOWN_QUE_MGR  				(CONFIG_FIRST_GEN_QUEUE + 2)   /* Teardown queue number */
#define TF_TEARDOWN_QUE_NUM  				  0


#define TF_LINKED_BUF_Q1           			(CONFIG_FIRST_GEN_QUEUE + 3)    /* First queue with attached buffers (for PA commands as well) */
#define TF_LINKED_BUF_Q1_BUF_SIZE  			128
#define TF_LINKED_BUF_Q1_NBUFS       		 24
extern UChar memQ1[TF_LINKED_BUF_Q1_NBUFS][TF_LINKED_BUF_Q1_BUF_SIZE];

#define TF_LINKED_BUF_Q2		   			(CONFIG_FIRST_GEN_QUEUE + 4)	/* Second queue with attached buffers */
#define TF_LINKED_BUF_Q2_BUF_SIZE  			256
#define TF_LINKED_BUF_Q2_NBUFS				 12
extern UChar memQ2[TF_LINKED_BUF_Q2_NBUFS][TF_LINKED_BUF_Q2_BUF_SIZE];

#define TF_LINKED_BUF_Q3		   			(CONFIG_FIRST_GEN_QUEUE + 5)
#define TF_LINKED_BUF_Q3_BUF_SIZE  			512
#define TF_LINKED_BUF_Q3_NBUFS      		  6
extern UChar memQ3[TF_LINKED_BUF_Q3_NBUFS][TF_LINKED_BUF_Q3_BUF_SIZE];

#define TF_LINKED_BUF_Q4		   			(CONFIG_FIRST_GEN_QUEUE + 6)
#define TF_LINKED_BUF_Q4_BUF_SIZE  		   4800
#define TF_LINKED_BUF_Q4_NBUFS      		  2
extern UChar memQ4[TF_LINKED_BUF_Q4_NBUFS][TF_LINKED_BUF_Q4_BUF_SIZE];

#define TF_HOST_LINKED_BUF_Q1           	(CONFIG_FIRST_GEN_QUEUE + 7)     /* First queue with attached buffers (for PA commands, Stats and Host Tx initiated packets ) */
#define TF_HOST_LINKED_BUF_Q1_BUF_SIZE  	128
#define TF_HOST_LINKED_BUF_Q1_NBUFS       	 24
extern UChar memHostQ1[TF_HOST_LINKED_BUF_Q1_NBUFS][TF_HOST_LINKED_BUF_Q1_BUF_SIZE];

#define TF_HOST_LINKED_BUF_Q2		   		(CONFIG_FIRST_GEN_QUEUE + 8)		/* Second queue with attached buffers */
#define TF_HOST_LINKED_BUF_Q2_BUF_SIZE  	256
#define TF_HOST_LINKED_BUF_Q2_NBUFS			 12
extern UChar memHostQ2[TF_HOST_LINKED_BUF_Q2_NBUFS][TF_HOST_LINKED_BUF_Q2_BUF_SIZE];

#define TF_HOST_LINKED_BUF_Q3		   		(CONFIG_FIRST_GEN_QUEUE + 9)
#define TF_HOST_LINKED_BUF_Q3_BUF_SIZE  	512
#define TF_HOST_LINKED_BUF_Q3_NBUFS      	  6
extern UChar memHostQ3[TF_HOST_LINKED_BUF_Q3_NBUFS][TF_HOST_LINKED_BUF_Q3_BUF_SIZE];

#define TF_HOST_LINKED_BUF_Q4		   		(CONFIG_FIRST_GEN_QUEUE + 10)
#define TF_HOST_LINKED_BUF_Q4_BUF_SIZE     4800
#define TF_HOST_LINKED_BUF_Q4_NBUFS       	  2
extern UChar memHostQ4[TF_HOST_LINKED_BUF_Q4_NBUFS][TF_HOST_LINKED_BUF_Q4_BUF_SIZE];


#define TF_FIRST_GEN_QUEUE		   			(CONFIG_FIRST_GEN_QUEUE + 11)		/* Queues available for general use */

 /* General purpose queue usage */
 #define Q_CMD_RECYCLE		  0		/* Command descriptors/buffers recycled here after sent to PA */
 #define Q_CMD_REPLY  		  1		/* Replies from PA routed here */
 #define Q_MATCH		  	  2		/* Packets from PA which match a lookup criteria */
 #define Q_NFAIL		      3		/* Packets from PA which matches a mac lookup, but failed an L3 lookup */
 #define Q_PARSE_ERR		  4		/* Packets which resulted in a parse error */
 #define Q_MULTI_0			  5		/* Multi route queue 0 */
 #define Q_MULTI_1			  6	    /* Multi route queue 1 */
 #define Q_MULTI_2			  7		/* Multi route queue 2 */
 #define Q_MULTI_3			  8		/* Multi route queue 3 */
 #define Q_MULTI_4			  9		/* Multi route queue 4 */
 #define Q_MULTI_5			 10		/* Multi route queue 5 */
 #define Q_MULTI_6           11		/* Multi route queue 6 */
 #define Q_MULTI_7			 12		/* Multi route queue 7 */
 
 #define Q_DATA_RECYCLE		 15	    /* DATA Buffer Recycle */
 
 #define Q_REQ_STATS_RECYCLE 16
 #define Q_STATS_REPLY		 17
 
 #define Q_PKT_RECV          18     /* The first Packet Receive Queue */
 #define Q_PKT_TX            19     /* The first Queue for Tx packet  */
 #define DEST_QUEUE_PKT_RECV        (TF_FIRST_GEN_QUEUE + Q_PKT_RECV)
 #define DEST_QUEUE_PKT_TX          (TF_FIRST_GEN_QUEUE + Q_PKT_TX)
 
/* The number of PA handles maintained by this test */
#define T2_NUM_LOCAL_HANDLES	64

 /* Commands to the PA are verified through the value in swinfo0.
  * The 16 ms bits are used as verification, the 16 lbs are for local handle id */
#define T2_CMD_SWINFO0_ADD_ID  		0x11110000  /* Identifies add mac command */
#define T2_CMD_SWINFO0_DEL_ID  		0x22220000  /* Identifies del mac command */
#define T2_CMD_SWINFO0_STATS_REQ_ID	0x33330000	/* Identifies the req stats command */
#define T2_CMD_SWINFO0_CONFIG_ID	0x44440000	/* Identifies the PA configuration command */
#define T2_CMD_SWINFO0_PKT_ID		0x55550000  /* Identifies the packet as a data packet */

/* SA related definitions */
#define SALLDSIM_SC_BUF_SIZE    320
#define SALLDSIM_NUM_SC_BUF     256     /* Multiple of 16 */ 

extern UInt8 salldsimScBuf[SALLDSIM_NUM_SC_BUF][SALLDSIM_SC_BUF_SIZE]; 

/* Memory to store test packets */
#define TEST_PKT_RAM_SIZE       0x20000
extern UInt8  testPktRam[TEST_PKT_RAM_SIZE];
extern UInt16 testPktOffset;

/* Define the test framework */
typedef struct tFramework_s  {
	
	Pa_Handle   passHandle;	/* PA instance handle */
	
	Cppi_Handle tfPaCppiHandle; /* PA CDMA handle */ 
	
	Cppi_ChHnd  tfPaTxChHnd[TF_PA_NUM_TX_CPDMA_CHANNELS];
	Cppi_ChHnd  tfPaRxChHnd[TF_PA_NUM_RX_CPDMA_CHANNELS];
	
	/* Queues */
	Int32 QPaTx[TF_PA_NUM_TX_CPDMA_CHANNELS];
	Int32 QfreeDesc;		      				/* Unassociated descriptor queue handle */
	Int32 QtDown;								/* Tear down queue handle */
	Int32 QLinkedBuf1;							/* First Queue with descriptors and attached linked buffers */
	Int32 QLinkedBuf2;							/* Second Queue with descriptors and attached linked buffers */
	Int32 QLinkedBuf3;							/* Third Queue with descriptors and attached linked buffers */
	Int32 QLinkedBuf4;							/* Fourth Queue with descriptors and attached linked buffers */
	Int32 QHostLinkedBuf1;						/* First Queue with descriptors and attached linked buffers */
	Int32 QHostLinkedBuf2;						/* Second Queue with descriptors and attached linked buffers */
	Int32 QHostLinkedBuf3;						/* Third Queue with descriptors and attached linked buffers */
	Int32 QHostLinkedBuf4;						/* Fourth Queue with descriptors and attached linked buffers */
	Int32 QGen[TF_NUM_GEN_QUEUES];				/* General purpose queues */
	
	Cppi_FlowHnd tfPaFlowHnd;					/* Flow handle */
	Int32		 tfFlowNum;						/* Physical flow number */
	
#ifdef USE_BIOS

	GateHwi_Handle gateHwi;		                /* HW interrupt disable handle */
    
	Semaphore_Handle tfPaTableL2Sem;			/* Semaphore for PA internal table for L2 */
	Semaphore_Handle tfPaTableL3Sem;			/* Semaphore for PA internal table for L3 */
	
	Semaphore_Handle tfQmSem;					/* Semaphore for queue manager */
	Semaphore_Handle tfSaSem;					/* Semaphore for SA LLD */
    
#endif    
    
    /* SA test related variables */
    Sa_Handle   salld_handle;
    int         salld_buf_sizes[sa_N_BUFS];
    salldSimChannel_t salldSimChn[salld_SIM_MAX_CHANNELS];
    
    /* Output files */
    FILE* fp_ctx;       /* context output file */
    FILE* fp_pktIn;     /* record packets which enter PA/SA */
    FILE* fp_pktOut;    /* record packets from PA/SA */
    
    /* record the current test */
	saTest_t* pTest;
} tFramework_t;

extern tFramework_t tFramework;

/* PA memory */
#define TF_PA_INST_SIZE	  128					            /* Required size = 88 */
extern UInt8 memPaInst[TF_PA_INST_SIZE];

#define TF_MAX_NUM_L2_HANDLES	64
#define TF_L2_TABLE_SIZE	(TF_MAX_NUM_L2_HANDLES * 32)	/* Requires 32 bytes per entry */
extern UInt8 memL2Ram[TF_L2_TABLE_SIZE];

#define TF_MAX_NUM_L3_HANDLES	128
#define TF_L3_TABLE_SIZE	(TF_MAX_NUM_L3_HANDLES * 72)	/* Requires 72 bytes per entry */
extern UInt8 memL3Ram[TF_L3_TABLE_SIZE];
	
/* QM memory */	
#define TF_NUM_DESC		 256    	/* 128 host descriptors managed by the QM */
#define TF_SIZE_DESC     128		/* Must be multiple of 16 */
#define TF_NUM_RES_DESC             (TF_LINKED_BUF_Q1_NBUFS         +  \
                                     TF_LINKED_BUF_Q2_NBUFS         +  \
                                     TF_LINKED_BUF_Q3_NBUFS         +  \
                                     TF_LINKED_BUF_Q4_NBUFS         +  \
                                     TF_HOST_LINKED_BUF_Q1_NBUFS    +  \
                                     TF_HOST_LINKED_BUF_Q2_NBUFS    +  \
                                     TF_HOST_LINKED_BUF_Q3_NBUFS    +  \
                                     TF_HOST_LINKED_BUF_Q4_NBUFS)

/* Memory used for the linking RAM and descriptor RAM */
//extern UInt64 memLinkRam[TF_NUM_DESC];
extern UInt8  memDescRam[TF_NUM_DESC * TF_SIZE_DESC];	


typedef UInt32  paStatsBmap_t; 

/* Stats numbers. Util functions are used to add these to the bit maps.
 * The values match the actual stats numbers */
#define TF_STATS_BM_C1_NUM_PACKETS		0
#define TF_STATS_BM_C1_TABLE_MATCH   	6
#define TF_STATS_BM_C1_NO_TABLE_MATCH   7
#define TF_STATS_BM_C1_VLAN_OVERFLOW    10
#define TF_STATS_BM_C1_NUM_MPLS			12
#define TF_STATS_BM_C1_PARSE_FAIL		13
#define TF_STATS_BM_C1_SILENT_DISCARD   17

/* 32 L2 handles are managed. This structure is used to track the handle and
 * the activation state state of the handle */
enum  {
  T2_L2_HANDLE_UNCONFIGURED = 0,
  T2_L2_HANDLE_PENDING_ACK,
  T2_L2_HANDLE_ACTIVE,
  T2_L2_HANDLE_DISABLED
};

typedef struct t2Handles_s  {

  paHandleL2L3_t  paHandle;     /* The handle returned by the PA LLD */

  UInt			state;		    /* T2_L2_HANDLE_UNCONFIGURED = handle not configured
							     * T2_L2_HANDLE_PENDING_ACK = handle configured and sent to pa
							     * T2_L2_HANDLE_ACTIVE = handle creation acknowledged by pa */

} t2Handles_t;

typedef struct t4Handles_s  {

  paHandleL4_t    paHandle;     /* The handle returned by the PA LLD */

  UInt			state;		    /* T2_L2_HANDLE_UNCONFIGURED = handle not configured
							     * T2_L2_HANDLE_PENDING_ACK = handle configured and sent to pa
							     * T2_L2_HANDLE_ACTIVE = handle creation acknowledged by pa */

} t4Handles_t;

typedef struct testPktDesc_s  {
    Bool    tx;                 /* 0: Tx Packet 1:Rx Packet */
    UInt16  chnum;              /* Channel Index */
    UInt16  size;               /* To be enhanced to include payload information */
    UInt16  payloadOffset;
    UInt16  payloadLen;
    UInt8*  pkt;                /* Pointer to the packet */
} testPktDesc_t;

extern paSysStats_t paTestExpectedStats;  /* Expected stats results */

#define SA_TEST_MAX_L2_HANDLES    4     /* Max number of L2 Handles */
#define SA_TEST_MAX_L3_HANDLES   30     /* Max number of L3 Handles */
#define SA_TEST_MAX_L4_HANDLES   90     /* Max number of L4 Handles */
 
extern t2Handles_t     l2Handles[SA_TEST_MAX_L2_HANDLES];
extern t2Handles_t     l3Handles[SA_TEST_MAX_L3_HANDLES];
extern t4Handles_t     l4Handles[SA_TEST_MAX_L4_HANDLES];
extern int numL2Handles, numL3Handles, numL4Handles;

/* 
 * Enable/Disable Simulator:
 *  
 * TRUE: simulator is used 
 * FALSE: silicon is used
 *
 * Note: This varaible is used to determine some test parameters such as number of test packets,
 *       number of test channels to avoid the long test time at simulator.
 */
extern Bool saSimTest;
extern UInt16 numTestPkt1;
extern UInt16 numTestPkt2;
extern UInt16 numTestPkt3;

/*
 * Assume that all L2/L3/L4 handles will be requested sequentially
 * and will last the entire test.
 * All the buffers will be initialized prior to each test.
 * Use simple MACROs for handle allocations
 */
#define TEST_ALLOC_L2_HANDLE()     (numL2Handles < SA_TEST_MAX_L2_HANDLES)?      \
                                   &l2Handles[numL2Handles++]:NULL
#define TEST_ALLOC_L3_HANDLE()     (numL3Handles < SA_TEST_MAX_L3_HANDLES)?      \
                                   &l3Handles[numL3Handles++]:NULL
#define TEST_ALLOC_L4_HANDLE()     (numL4Handles < SA_TEST_MAX_L4_HANDLES)?      \
                                   &l4Handles[numL4Handles++]:NULL

/* Macros to form the packet info structure */
#define TF_FORM_PKT_INFO_WORD0(cmdId,recordLen,startOffset)  ((cmdId << 29) | (recordLen << 24) | startOffset)
#define TF_FORM_PKT_INFO_WORD1(endOffset,errIdx,pmatch,c2c,l1PdspId,l1Idx)  ((endOffset << 16) | (errIdx << 11) | (pmatch << 10) | \
																			 (c2c << 9) | (l1PdspId << 6) | (l1Idx))
#define TF_FORM_PKT_INFO_WORD2(l3Offset,l4Offset,l5Offset,ahEspOffset)		((l3Offset << 24) | (l4Offset << 16) | (l5Offset << 8) | ahEspOffset)
#define TF_FORM_PKT_INFO_WORD3(hdrBitmask,nextHdr,vCount,ipCount,greCount,frag,route,ipmrt)  \
								((hdrBitmask << 21) | (nextHdr << 16) | (vCount << 12) | (ipCount << 8) | (greCount << 4) | \
								 (frag << 3) | (route << 2) | (ipmrt << 1))
#define TF_FORM_PKT_INFO_WORD4(pseudo)  (pseudo << 16)

#define MK_UINT16(a, b)             (((a) << 8) | (b))
#define MK_UINT32(a, b, c, d)       (((a) << 24) | ((b) << 16) | ((c) << 8) | (d))




#define SYS_DIV_ROUND_UP(val,rnd)   (((val)+(rnd)-1)/(rnd))
#define SYS_ROUND_UP(val,rnd)       SALLD_DIV_ROUND_UP(val,rnd)*(rnd)

#ifndef USE_BIOS
#ifndef ARM11
/* Inline intrinsic, to save mips in data path */
#define thwCriticalBeginFast() _disable_interrupts()

/* Type to save from _disable_interrupts, and to pass to _restore_interrupts */
typedef UInt32 thwCriticalState_t;

/* Function, to save memory in control code */
void thwCriticalBegin(void);

/******************************************************************************
 * FUNCTION PURPOSE: End of critical section (up to 15 sections can be nested)
 *                   
 ******************************************************************************
 * DESCRIPTION: Restore interrupt enable/disable status encountered on entry to
 *              critical section. Bit ST1_INTM is poped from the global variable.
 *****************************************************************************/
#define thwCriticalEndFast(x) _restore_interrupts(x)
void thwCriticalEnd(void);
#endif /* ARM11 */
#endif /* USE_BIOS */

#undef L2_CACHE
#ifdef L2_CACHE

#define SYS_CACHE_LINE_SIZE             128

    /* Invalidate L2 cache. This should invalidate L1D as well. 
     * Wait until operation is complete. */    
#define SYS_CACHE_INV(addr, size, code)    CACHE_invL2 (addr, (size) + ((uint32_t)(addr) & (SYS_CACHE_LINE_SIZE - 1)), code)

    /* Writeback L2 cache. This should Writeback L1D as well. 
     * Wait until operation is complete. */ 
#define SYS_CACHE_WB(addr, size, code)     CACHE_wbL2 (addr, (size) + ((uint32_t)(addr) & (SYS_CACHE_LINE_SIZE - 1)), code)


#else       

#define SYS_CACHE_LINE_SIZE              64

    /* Invalidate L1D cache and wait until operation is complete. 
     * Use this approach if L2 cache is not enabled */    
#define SYS_CACHE_INV(addr, size, code)    CACHE_invL1d (addr, (size) + ((uint32_t)(addr) & (SYS_CACHE_LINE_SIZE - 1)), code)
    /* Writeback L1D cache and wait until operation is complete. 
     * Use this approach if L2 cache is not enabled */    
#define SYS_CACHE_WB(addr, size, code)     CACHE_wbL1d (addr, (size) + ((uint32_t)(addr) & (SYS_CACHE_LINE_SIZE - 1)), code)

#define SYS_CACHE_INV1(addr, size, code)    CACHE_invL1d (addr, (size), code)
#define SYS_CACHE_WB1(addr, size, code)     CACHE_wbL1d (addr, (size), code)


#endif  /* L2_CACHE */
#ifndef SALLD_UNIT_TEST_MAX_SEGMENTS
#define SALLD_UNIT_TEST_MAX_SEGMENTS 10
#endif
/* Prototypes */

/* salld.c: pktutl ported functions */
/******************************************************************************
 * FUNCTION PURPOSE: Read 8 bit value from 16 bit word (macro version)
 ******************************************************************************
 * DESCRIPTION: Returns 8 bit value from 16 bit word.  Assumes nothing.
 *
 * uint16_t pktRead8bits_m (
 *    uint8_t *base,       - Base of byte array
 *    uint16_t byteOffset); - Byte offset to read
 *
 *****************************************************************************/
static inline uint16_t pktRead8bits_m (uint8_t *base, uint16_t byteOffset)
{
  char *src = (char *)base;
  char wvalue = *(src + byteOffset);
  uint16_t readByte = (uint16_t)(wvalue & 0xFF);
  return readByte;
} /* pktRead8bits_m */

/******************************************************************************
 * FUNCTION PURPOSE: Write 8 bit value into 16 bit word (macro version)
 ******************************************************************************
 * DESCRIPTION: Writes 8 bit value into 16 bit word; nothing assumed.
 *
 * void pktWrite8bits_m (
 *    uint8_t *base,      - Base of byte array
 *    uint16_t byteOffset, - byte offset to write
 *    uint16_t val)        - Byte in low 8 bits of val
 *
 *****************************************************************************/
static inline void pktWrite8bits_m (uint8_t *base, uint16_t byteOffset, uint16_t val)
{
  char *wptr = ((char *)base + byteOffset);
  *wptr = (char)(val & 0xFF);
} /* pktWrite8bits_m */

/******************************************************************************
 * FUNCTION PURPOSE: Write 16 bit value into 16 bit word (macro version)
 ******************************************************************************
 * DESCRIPTION: Writes 16 bit value into 16 bit word.  No assumptions
 *
 * void pktWrite16bits_m (
 *    uint8_t *base,      - Base of byte array
 *    uint16_t byteOffset, - byte offset to write; assumed to be even
 *    uint16_t val)        - 16 bit val
 *
 *****************************************************************************/
static inline void pktWrite16bits_m (uint8_t *base, uint16_t byteOffset, uint16_t val)
{
  char *wptr = ((char *)base + byteOffset);

  /* Shift/mask is endian-portable, but look out for stupid compilers */
  wptr[0] = (char)(val>>8);
  wptr[1] = (char)(val & 0xff);

} /* pktWrite16bits_m */

/******************************************************************************
 * FUNCTION PURPOSE: Read 16 bit value from 16 bit word (macro version)
 ******************************************************************************
 * DESCRIPTION: Returns 16 bit value from 16 bit word.  No assumptions.
 *
 * uint16_t pktRead16bits_m (
 *    uint8_t *base,       - Base of byte array
 *    uint16_t byteOffset); - Byte offset to read, assumed to be even
 *
 *****************************************************************************/
static inline uint16_t pktRead16bits_m (uint8_t *base, uint16_t byteOffset)
{
  char *wptr = ((char *)base + byteOffset);
  uint16_t ret;

  /* Shift/mask is endian-portable, but look out for stupid compilers */
  ret = (((uint16_t)wptr[0]) << 8) | (wptr[1] & 0xFF);

  return ret;
} /* pktRead16bits_m */

/******************************************************************************
 * FUNCTION PURPOSE: Write 32 bit value into 16 bit words (macro version)
 ******************************************************************************
 * DESCRIPTION: Writes 32 bit value into 16 bit word; No
 *              alignment assumed
 *
 * void pktWrite32bits_m (
 *    uint8_t *base,      - Base of byte array
 *    uint16_t byteOffset, - byte offset to write; assumed to be even.
 *    uint32_t val)       - 32 bit val
 *
 *****************************************************************************/
static inline void pktWrite32bits_m (uint8_t *base, uint16_t byteOffset, uint32_t val)
{
  /* Shift/mask is endian-portable, but look out for stupid compilers */
  pktWrite16bits_m (base, byteOffset, (uint16_t)(val>>16));
  pktWrite16bits_m (base, byteOffset+2, (uint16_t)(val&0xffff));

} /* pktWrite32bits_m */

/******************************************************************************
 * FUNCTION PURPOSE: Read 32 bit value from 16 bit words (macro version)
 ******************************************************************************
 * DESCRIPTION: Read 32 bit value from 16 bit words; No
 *              alignment assumed
 *
 * uint32_t pktRead32bits_m (
 *    uint8_t *base,      - Base of byte array
 *    uint16_t byteOffset) - byte offset to write; assumed to be even.
 *
 *****************************************************************************/
static inline uint32_t pktRead32bits_m (uint8_t *base, uint16_t byteOffset)
{
  uint32_t ret;

  /* Shift/mask is endian-portable, but look out for stupid compilers */
  ret = (((uint32_t)pktRead16bits_m (base, byteOffset)) << 16);
  ret |= (uint32_t)pktRead16bits_m (base, byteOffset + 2);

  return ret;
} /* pktRead32bits_m */

Int setupTestFramework (Void);
Int verifyTestFramework (Void);
Void saTestRecoverAndExit (tFramework_t *tf, saTest_t *pat, saTestStatus_t testResult);
Void utilCycleDelay (Int count);
UInt32 utilgAddr(UInt32 x);
void utilInputTestPkts (char* file_name);
UInt16 utilOnesCompAdd (UInt16 value1, UInt16 value2);
UInt16 utilOnesCompChkSum ( UInt16 *p_data, UInt16 len );
Bool utilGetPkt(testPktDesc_t *pktDesc);  
void utilSetIpv4ChkSum (UInt8 *data);
UInt16 utilGetIpv4PsudoChkSum (UInt8 *data, UInt16 payloadLen);
UInt16 utilGetIpv6PsudoChkSum (UInt8 *data, UInt16 payloadLen);
void utilSetUdpChkSum (UInt8 *data, UInt16 len, UInt16 psudoChkSum);
Void utilPrepFout(tFramework_t  *tf);
Void utilCloseFout(tFramework_t  *tf);
Void utilOutputPkt(tFramework_t  *tf, UInt32 *swInfo, UInt32 psInfoSize, UInt32* psInfo,
                   UInt32 pktSize, UInt8* pkt, Bool in);
Void utlOutputSc (tFramework_t *tf,  UInt16 scSize, UInt16 scID, UInt8* scBuf);
Void utilOutputScFromSwInfo(tFramework_t *tf, UInt32* pSwInfo);
Void utilDispKeyStream(UInt8* buf, UInt16 keySize, UInt16 blockSize);


Cppi_HostDesc*  testCommonGetBuffer(tFramework_t *tf, Int size);
Int testCommonRequestPaStats (char *fname, tFramework_t *tf, Bool reset, Int32 QSource, Int32 QRecycle,  paCmdReply_t *reply);
Int testCommonCompareStats (char *fname, paSysStats_t *expected, paSysStats_t *actual);
Int testCommonRecycleLBDesc (tFramework_t *tf, Cppi_HostDesc *hd);
Int testCommonRecycleHostLBDesc (tFramework_t *tf, Cppi_HostDesc *hd);
Int testCommonComparePktInfo (Char *tfName, pasahoLongInfo_t *expected, pasahoLongInfo_t *actual);
Void testCommonIncStats (paStatsBmap_t *map,  paSysStats_t *stats);
Bool testCommonReqAndChkPaStats (tFramework_t *tf, saTest_t *pat, paSysStats_t *expected);
Bool testCommonAddMac (tFramework_t *tf, saTest_t *pat, paEthInfo_t *ethInfo, paRouteInfo_t *matchRoute, paRouteInfo_t *nfailRoute, 
 	                   paHandleL2L3_t *pl2handle, Int32 QCmdMem,  paCmdReply_t *repInfo);
Bool testCommonAddIp  (tFramework_t *tf, saTest_t *pat, paIpInfo_t *ipInfo, paRouteInfo_t *matchRoute, paRouteInfo_t *nfailRoute, 
 	                   paHandleL2L3_t l2handle, paHandleL2L3_t *pl3handle, Int32 QCmdMem, paCmdReply_t *repInfo);
Bool testCommonAddIp2 (tFramework_t *tf, saTest_t *pat, int lutInst, paIpInfo_t *ipInfo, paRouteInfo_t *matchRoute, paRouteInfo_t *nfailRoute, 
 	                   paHandleL2L3_t l2handle, paHandleL2L3_t *pl3handle, Int32 QCmdMem, paCmdReply_t *repInfo);
Bool testCommonAddUdp (tFramework_t *tf, saTest_t *pat, UInt16 port, Bool replace, paRouteInfo_t *routeInfo,  
 	                   paHandleL2L3_t l3handle, paHandleL4_t l4handle, Int32 QCmdMem, paCmdReply_t *repInfo);
Bool testCommonConfigExceptionRoute (tFramework_t *tf, saTest_t *pat, int nRoute, int *routeTypes, paRouteInfo_t *eRoutes, 
 	                       		     Int32 QCmdMem, paCmdReply_t *repInfo);
Bool testCommonGlobalConfig (tFramework_t *tf, saTest_t *pat, paCtrlInfo_t *cfgInfo, Int32 QCmdMem, paCmdReply_t *repInfo);
Bool testCommonDelHandle (tFramework_t *tf, saTest_t *pat, paHandleL2L3_t *paHdl, Int QCmdMem, paCmdReply_t *cmdReply);
Bool testCommonDelL4Handle (tFramework_t *tf, saTest_t *pat, paHandleL4_t paHdl, Int QCmdMem, paCmdReply_t *cmdReply);
Bool testCommonDelL2L3Connection (tFramework_t *tf, saTest_t *pat, Int numHandles, t2Handles_t *l2Handles, Int *numConnet);
Bool testCommonDelL4Connection (tFramework_t *tf, saTest_t *pat, Int numHandles, t4Handles_t *l4Handles, Int *numConnet);
void testCommonSendNullPkt(tFramework_t *tf, UInt32 *swInfo);
 
void testDispPkts (Cppi_HostDesc *hd);
 	
Void mdebugHaltPdsp (Int pdspNum);

/* Tests */
Void saESPTest (UArg a0, UArg a1);
Void saAHTest (UArg a0, UArg a1);
Void saESPNATTTest (UArg a0, UArg a1);
Void saSRTPTest (UArg a0, UArg a1);
Void saSRTCPTest (UArg a0, UArg a1);
Void saESPMultiSgmtTest (UArg a0, UArg a1);
Void saSRTPMultiSgmtTest (UArg a0, UArg a1);
Void saSRTCPMultiSgmtTest (UArg a0, UArg a1);
Void saAHMultiSgmtTest (UArg a0, UArg a1);
Void saSrtpEspTest (UArg a0, UArg a1);
Void saSrtpAhTest (UArg a0, UArg a1);
Void saAirCipheringTest (UArg a0, UArg a1);
Void saAirCipheringKeyGenTest (UArg a0, UArg a1);
Void saDataModeTest (UArg a0, UArg a1);
Void saESPTest2 (UArg a0, UArg a1);
Void saESPReplayTest (UArg a0, UArg a1);
Void saAHReplayTest (UArg a0, UArg a1);
Void saSRTPReplayTest (UArg a0, UArg a1);
Void saSRTCPReplayTest (UArg a0, UArg a1);
Void saSRTPRekeyTest1 (UArg a0, UArg a1);
Void saSRTPRekeyTest2 (UArg a0, UArg a1);
Void saSRTPRekeyTest3 (UArg a0, UArg a1);
Void saSRTPReKeyTest4 (UArg a0, UArg a1);
Void saSRTCPRekeyTest1 (UArg a0, UArg a1);
Void saSRTCPRekeyTest2 (UArg a0, UArg a1);
Void saESPErrTest (UArg a0, UArg a1);
Void saAHErrTest (UArg a0, UArg a1);
Void saSRTPErrTest (UArg a0, UArg a1);
Void saSRTCPErrTest (UArg a0, UArg a1);
Void saRngTest (UArg a0, UArg a1);
Void saPkaTest (UArg a0, UArg a1);
#endif /* _UNITTEST_H */
