/**
 *   @file  traceframework.h
 *
 *   @brief   
 *      This has all necessary include files needed for the appliation to include to trace framework.
 *
 *  ============================================================================
 *  @n   (C) Copyright 2012, Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef __TRACEFRAMEWORK_H_
#define __TRACEFRAMEWORK_H_

#ifdef __cplusplus
extern "C" {
#endif

/** 
 * @mainpage  Trace Framework
 *
 * Trace framework is a framework which binds producer and consumers via
 * a contract memory (shared memory between producers and consumers)
 *
 * @section Introduction Introduction
 *
 * In a multicore environment there would be data generated by a producer and would need
 * to be consumed by multiple consumers.
 * Example: UIA log information to be consumed by ARM and System Analyzer.
 * The framework mentioned in this document is intended to provide a general framework
 * for a system information source (producer) to provide data to multiple consumers in the
 * system. 
 * This needs some kind of synchronization between consumers and producer providing the
 * information data.
 * The synchronization is done via the contract memory.
 * Producer populates the Ring buffer and ring consumers consume the ring buffer and send
 * the data to external actual consumers (like System Analyzer in CCS).
 * Each core can have multiple consumer Instances and Producer Instances. Each Producer
 * instance would be tied to a ring buffer and many consumer instances can consume data
 * from a ring buffer. Every consumer needs to have its own channel to send the data out,
 * which triggers the draining of the ring buffer
 *
 * @section General_Framework General Framework
 *
 * @subsection Ring_Producer Ring Producer
 *
 * The ring producer produces the trace/log information in a ring buffer and notifies the
 * consumers hooked to that ring when the ring buffers are ready. The producer fills up the
 * information in the ring buffer sequentially, servicing the slowest possible consumer for
 * that ring. So, a slowest consumer in the system can potentially make the fast consumers
 * to drop messages.
 * The producers and consumers handshake through a shared memory region called as Contract Memory.
 *
 * @subsection Ring_Consumer Ring Consumer
 * 
 * The ring consumer reads the ring buffer from the producer and outputs it to the
 * appropriate transport channel. The consumer updates the necessary control messages to
 * the producer via the Contract.
 *
 * @subsection Ring_Contract Contract
 *
 * Contract is a shared memory area between the producer and multiple consumers. 
 * Traceframework supports two contract versions.
 * - Contract Version 1: All the contracts in the system are in one contiguous big chunk of the memory. Either DSP or ARM needs to 
 * create that contiguous pool and needs to initialize it via trace framework provided APIs. 
 *   - @ref tf_contractInit - to initialize the contract memory pool
 *   - @ref tf_contractCreate - to create the named contract 
 *   - @ref tf_contractFind - to find the contract by name in producer/consumer functions. 
 * Please note that the contracts created under this version are named contracts. Traceframework internally handles the name of the contracts.
 *   - Limitations:
 *      - Requires big chunk of contiguous memory
 *      - When ARM initializes the Contract Pool (concept of virtual memory), DSPs do not get to know the physical base address of the contract
 * of the contracts. Hence every contract created under version 1 
 * - Contract Version 2: Handling of name within the contracts are removed. The association of name to the contract handle needs to be done
 * outside traceframework library. There is no need to pre-allocate the big contract chunk of memory. Contracts can be created as and when needed
 * on either DSP or ARM using the below traceframework provided APIs.
 *   - @ref tf_newContract - to initialize, create the new contract
 *   
 * @verbatim
	 The layout for the contract memory is as below.
          
     +-------+----------------------------------------------------------------------------------------------------------------+
     | Line1 |   Contract name (128 Bytes)                                                                                    |
     +-------+-----------------+--------------------+-------------------+-------------------+--------------+------------------+
     | Line2 | Type (4 bytes)  | Contract Version   | Ring Buf Base     | Num of Ring Bufs  | Ring Buf Size| Padding          |
     |       |                 | (4 Bytes)          | (4 bytes)         | (4 Bytes)         |  (4 bytes)   | (112Bytes)       |
     +-------+-----------------+--------------------+-------------------+-------------------+--------------+------------------+
     | Line3 | producer current|                                                                                              |
     |       | index (4 bytes) | Padding (124 bytes)                                                                          |
     +-------+-----------------+----------------------------------------------------------------------------------------------+
     | Line4 | global last free|                                                                                              |
     |       | index (4bytes)  | Padding (124 bytes)                                                                          |
     +-------+-----------------+----------------------------------------------------------------------------------------------+
     | Line5 | Number of Consu |                                                                                              |
     |       | -ers (4 bytes)  | Padding (124 bytes)                                                                          |
     +-------+-----------------+--------------------+-------------------+--------------------+--------------------------------+
     | Line6 | Cons1MagicID    | Cons1_last_free    | Cons1_Notify Reg  | Cons1 Notfy Method | Cons1 Local CallBk| Padding for|
     |       | (4 bytes)       | Buf Index (4 Bytes)| Mask (4 bytes)    | (4 Bytes)          | Function (4 Bytes)| 104 bytes  |
     +-------+-----------------+--------------------+-------------------+--------------------+--------------------------------+
     | Line7 | Cons2MagicID    | Cons2_last_free    | Cons2_Notify Reg  | Cons2 Notfy Method | Cons2 Local CallBk| Padding for|
     |       | (4 bytes)       | Buf Index (4 Bytes)| Mask (4 bytes)    | (4 Bytes)          | Function (4 Bytes)| 104 bytes  |
     +-------+-----------------+--------------------+-------------------+--------------------+--------------------------------+
     | Line8 | Cons3MagicID    | Cons3_last_free    | Cons3_Notify Reg  | Cons3 Notfy Method | Cons3 Local CallBk| Padding for|
     |       | (4 bytes)       | Buf Index (4 Bytes)| Mask (4 bytes)    | (4 Bytes)          | Function (4 Bytes)| 104 bytes  |
     +-------+-----------------+--------------------+-------------------+--------------------+--------------------------------+
     | Line9 | Cons4MagicID    | Cons4_last_free    | Cons4_Notify Reg  | Cons4 Notfy Method | Cons4 Local CallBk| Padding for|
     |       | (4 bytes)       | Buf Index (4 Bytes)| Mask (4 bytes)    | (4 Bytes)          | Function (4 Bytes)| 104 bytes  |
     +-------+-----------------+--------------------+-------------------+--------------------+--------------------------------+
     | Line10|   Out of Band data Information (128 Bytes)                                                                     |
     +-------+----------------------------------------------------------------------------------------------------------------+

 Contract Name Field
    Line 1 in the contract is reserved for the name of the contract. This filed is applicable only for Contract version 1.
    Few examples of the contract name are,
    1. UIA_CONTRACT_CORE0,
    2. UIA_CONTRACT_CORE1,
    3. UIA_CONTRACT_CORE2,
    4. UIA_CONTRACT_CORE3.

 Producer Information Fields
    Line 2 through line 9 information is used by the ring producer to fill up the ring buffers
    and to notify the attached consumers in the ring. The contract supports maximum of 4
    consumers to be attached simultaneously to the ring.

 Consumer Information Fields
    Line 2 through to line 5 is used by every consumer to get the information about the
    current index in the ring buffer where the producer is filling up. Every consumer in the
    contract has its own dedicated Cache line information updated each time the ring buffer is
    consumed. The dedicated consumer information is read by other consumers on the
    contract which influences the producers head index to fill up the ring buffer.

 Limitations
    1. The contract and ring buffers need to be aligned to cache boundary and to be
        placed under shared memory.
    2. Producer ring needs to be created first before any consumers are instantiated for
        the ring.
    3. Every Consumer instance needs to have dedicated unique transport channel.
    4. Every producer for the ring needs to be notified for any new consumer in the
        contract.
  @endverbatim
 *
 * @subsection Assumptions Assumptions
 * - The contract memory is always located in a shared memory for all the cores.
 * - The elements in the contract memory are cache line aligned.
 * 
 * @subsection Rebuilding Rebuilding the Libraries
 * Please follow the below steps to rebuild the libraries 
 * - for DSP side
 *   - Setup the build environment (sample environment setting is provided as in pdk build environment)
 *   - cd "\ti\pdk_##_1_00_##_##\packages"
 *   - gmake -C ti / instrumentation / traceframework LIBDIR=./lib clean
 *   - gmake -C ti / instrumentation / traceframework LIBDIR=./lib all
 * - for ARM side
 *   - cd "\ti\pdk_##_1_00_##_##\packages"
 *   - export DEVICE=device (e.g., k2h)
 *   - export CROSS_TOOL_INSTALL_PATH=\<croos_tool_install_path\> (e.g., /opt/linaro-2013.03/bin)
 *   - export CROSS_TOOL_PRFX= \<cross_tool_prefix\> (e.g., for linaro 2013.03 arm-linux-gnueabihf-)
 *   - export CUIA_INSTALL_DIR=\<where cuia install can be found\> (e.g., devkit base path or actual cuia install location like $HOME/cuia_1_00_00_13)
 *   - export PDK_INSTALL_PATH=\<current_working_directory\>
 *   - make -C ti/instrumentation/traceframework -f makefile_armv7 clean all
 */

#include <string.h>
/* OSAL Includes */
#include <ti/instrumentation/traceframework/contract_osal.h>
#include <ti/instrumentation/traceframework/consumer_osal.h>
#include <ti/instrumentation/traceframework/producer_osal.h>

/* Producer/Consumer Library includes */
#include <ti/instrumentation/traceframework/trace_contract.h>
#include <ti/instrumentation/traceframework/producer.h>
#include <ti/instrumentation/traceframework/consumer.h>

/**
@defgroup TRACEFRAMEWORK_SYMBOL  Trace Framework Symbols
@ingroup TRACEFRAMEWORK_API
*/

/**
@defgroup TRACEFRAMEWORK_ENUM  Trace Framework Enumerations
@ingroup TRACEFRAMEWORK_API
*/

/**
@defgroup TRACEFRAMEWORK_DATASTRUCT  Trace Framework Data Structures
@ingroup TRACEFRAMEWORK_API
*/

/**
@defgroup TRACEFRAMEWORK_FUNCTIONS  Trace Framework Functions
@ingroup TRACEFRAMEWORK_API
*/

/**
@defgroup TRACEFRAMEWORK_UIA_FUNCTIONS  Trace Framework Functions (UIA, for DSP side only)
@ingroup TRACEFRAMEWORK_API
*/

/**
@defgroup TRACEFRAMEWORK_CUIA_FUNCTIONS  Trace Framework Functions (CUIA, for ARM side only)
@ingroup TRACEFRAMEWORK_API
*/

/**
@defgroup TRACEFRAMEWORK_OSAL  Trace Framework Operating System Abstraction Layer (OSAL) Functions 
@ingroup TRACEFRAMEWORK_API
*/

/** @addtogroup TRACEFRAMEWORK_FUNCTIONS
@{
*/

/**
 * ============================================================================
 *  @n@b tf_consumerSetProducerState
 *
 *  @b  brief
 *  @n  API to set the producer state to either streaming, free run or freeze
 *
 *  @param[in]  cHandle
 *      consumer handle
 *  @param[in]  state
 *      intended producer state change
 *
 *  @return
 *      Status of the operation
 *
 * =============================================================================
 */
extern tf_consStatus_e tf_consumerSetProducerState(tf_cons_HANDLE cHandle, tf_producerState_e state);

/**
 * ============================================================================
 *  @n@b tf_consumerGetProducerState
 *
 *  @b  brief
 *  @n  API to sample the current producer state
 *
 *  @param[in]  cHandle
 *      consumer handle
 *
 *  @return
 *      Current sampled producer state in the contract
 *
 * =============================================================================
 */
extern tf_producerState_e tf_consumerGetProducerState(tf_cons_HANDLE cHandle);
/**
@}
*/


#ifdef __cplusplus
}
#endif

#endif /* __TRACEFRAMEWORK_H_ */
/* Nothing past this point */    
